@@ -75,9 +75,12 @@ int hexbin2(double width, double x, double y,
 
     z = -x - y;
 
-    ix = rx = floor(x + 0.5);
-    iy = ry = floor(y + 0.5);
-    iz = rz = floor(z + 0.5);
+    rx = floor(x + 0.5);
+    ix = (int)rx;
+    ry = floor(y + 0.5);
+    iy = (int)ry;
+    rz = floor(z + 0.5);
+    iz = (int)rz;
 
     s = ix + iy + iz;
 
@@ -156,7 +159,7 @@ struct snyder_constants {
 };
 
 /* TODO put these in radians to avoid a later conversion */
-ISEA_STATIC
+ISEA_STATIC const
 struct snyder_constants constants[] = {
     {23.80018260, 62.15458023, 60.0, 3.75, 1.033, 0.968, 5.09, 1.195, 1.0},
     {20.07675127, 55.69063953, 54.0, 2.65, 1.030, 0.983, 3.59, 1.141, 1.027},
@@ -167,10 +170,6 @@ struct snyder_constants constants[] = {
     {37.37736814, 36.0, 30.0, 17.27, 1.163, 0.860, 13.14, 1.584, 1.0},
 };
 
-#define E 52.62263186
-#define F 10.81231696
-
-#define DEG60 1.04719755119659774614
 #define DEG120 2.09439510239319549229
 #define DEG72 1.25663706143591729537
 #define DEG90 1.57079632679489661922
@@ -214,7 +213,7 @@ static int      tri_v1[] = {0, 0, 0, 0, 0, 0, 6, 7, 8, 9, 10, 2, 3, 4, 5, 1, 11,
 #define F_RAD 0.18871053072122403508
 
 /* triangle Centers */
-struct isea_geo icostriangles[] = {
+static const struct isea_geo icostriangles[] = {
     {0.0, 0.0},
     {-DEG144, E_RAD},
     {-DEG72, E_RAD},
@@ -263,14 +262,12 @@ az_adjustment(int triangle)
 /* H = 0.25 R tan g = */
 #define TABLE_H 0.1909830056
 
-#define RPRIME 0.91038328153090290025
-
 ISEA_STATIC
 struct isea_pt
 isea_triangle_xy(int triangle)
 {
     struct isea_pt  c;
-    double Rprime = 0.91038328153090290025;
+    const double Rprime = 0.91038328153090290025;
 
     triangle = (triangle - 1) % 20;
 
@@ -314,6 +311,12 @@ sph_azimuth(double f_lon, double f_lat, double t_lon, double t_lat)
     return az;
 }
 
+#ifdef _MSC_VER
+#pragma warning( push )
+/* disable unreachable code warning for return 0 */
+#pragma warning( disable : 4702 )
+#endif
+
 /* coord needs to be in radians */
 ISEA_STATIC
 int
@@ -485,19 +488,21 @@ isea_snyder_forward(struct isea_geo * ll, struct isea_pt * out)
     exit(EXIT_FAILURE);
 
     /* not reached */
-    return 0;       /* supresses a warning */
+    return 0;       /* suppresses a warning */
 }
 
+#ifdef _MSC_VER
+#pragma warning( pop )
+#endif
+
 /*
- * return the new coordinates of any point in orginal coordinate system.
- * Define a point (newNPold) in orginal coordinate system as the North Pole in
+ * return the new coordinates of any point in original coordinate system.
+ * Define a point (newNPold) in original coordinate system as the North Pole in
  * new coordinate system, and the great circle connect the original and new
  * North Pole as the lon0 longitude in new coordinate system, given any point
- * in orginal coordinate system, this function return the new coordinates.
+ * in original coordinate system, this function return the new coordinates.
  */
 
-#define PRECISION 0.0000000000005
-
 /* formula from Snyder, Map Projections: A working manual, p31 */
 /*
  * old north pole at np in new coordinates
@@ -892,7 +897,7 @@ int isea_disn(struct isea_dgg *g, int quad, struct isea_pt *di) {
         sn += 2;
     } else {
         sidelength = (int) (pow(g->aperture, g->resolution / 2.0) + 0.5);
-        sn = (quad - 1) * hexes + sidelength * di->x + di->y + 2;
+        sn = (int) ((quad - 1) * hexes + sidelength * di->x + di->y + 2);
     }
 
     g->serial = sn;
@@ -908,8 +913,11 @@ ISEA_STATIC
 int isea_hex(struct isea_dgg *g, int tri,
         struct isea_pt *pt, struct isea_pt *hex) {
     struct isea_pt v;
+#ifdef FIXME
     int sidelength;
-    int d, i, x, y, quad;
+    int d, i, x, y;
+#endif
+    int quad;
 
     quad = isea_ptdi(g, tri, pt, &v);
 
@@ -917,9 +925,9 @@ int isea_hex(struct isea_dgg *g, int tri,
     hex->y = v.y;
 
     return 1;
-
-    d = v.x;
-    i = v.y;
+#ifdef FIXME
+    d = (int)v.x;
+    i = (int)v.y;
 
     /* Aperture 3 odd resolutions */
     if (g->aperture == 3 && g->resolution % 2 != 0) {
@@ -961,6 +969,7 @@ int isea_hex(struct isea_dgg *g, int tri,
     }
 
     return 1;
+#endif
 }
 
 ISEA_STATIC
@@ -1017,7 +1026,9 @@ isea_forward(struct isea_dgg *g, struct isea_geo *in)
  */
 
 #define PJ_LIB__
-#include <projects.h>
+#include <errno.h>
+#include <proj.h>
+#include "projects.h"
 
 PROJ_HEAD(isea, "Icosahedral Snyder Equal Area") "\n\tSph";
 
@@ -1044,27 +1055,11 @@ static XY s_forward (LP lp, PJ *P) {           /* Spheroidal, forward */
 }
 
 
-static void *freeup_new (PJ *P) {                       /* Destructor */
-    if (0==P)
-        return 0;
-    if (0==P->opaque)
-        return pj_dealloc (P);
-
-    pj_dealloc (P->opaque);
-    return pj_dealloc(P);
-}
-
-static void freeup (PJ *P) {
-    freeup_new (P);
-    return;
-}
-
-
 PJ *PROJECTION(isea) {
     char *opt;
     struct pj_opaque *Q = pj_calloc (1, sizeof (struct pj_opaque));
     if (0==Q)
-        return freeup_new (P);
+        return pj_default_destructor (P, ENOMEM);
     P->opaque = Q;
 
 
@@ -1082,7 +1077,7 @@ PJ *PROJECTION(isea) {
         } else if (!strcmp(opt, "pole")) {
             isea_orient_pole(&Q->dgg);
         } else {
-            E_ERROR(-34);
+            return pj_default_destructor(P, PJD_ERR_ELLIPSOID_USE_REQUIRED);
         }
     }
 
@@ -1121,7 +1116,7 @@ PJ *PROJECTION(isea) {
         }
         else {
             /* TODO verify error code.  Possibly eliminate magic */
-            E_ERROR(-34);
+            return pj_default_destructor(P, PJD_ERR_ELLIPSOID_USE_REQUIRED);
         }
     }
 
@@ -1144,33 +1139,3 @@ PJ *PROJECTION(isea) {
     return P;
 }
 
-
-#ifndef PJ_SELFTEST
-int pj_isea_selftest (void) {return 0;}
-#else
-
-int pj_isea_selftest (void) {
-    double tolerance_lp = 1e-10;
-    double tolerance_xy = 1e-7;
-
-    char s_args[] = {"+proj=isea   +a=6400000    +lat_1=0.5 +lat_2=2"};
-
-    LP fwd_in[] = {
-        { 2, 1},
-        { 2,-1},
-        {-2, 1},
-        {-2,-1}
-    };
-
-    XY s_fwd_expect[] = {
-        {-1097074.9480224741, 3442909.3090371834},
-        {-1097074.9482647954, 3233611.7285857084},
-        {-1575486.3536415542, 3442168.3420281881},
-        {-1575486.353880283,  3234352.6955947056},
-    };
-
-    return pj_generic_selftest (0, s_args, tolerance_xy, tolerance_lp, 4, 4, fwd_in, 0, s_fwd_expect, 0, 0, 0);
-}
-
-
-#endif
