@@ -1,5 +1,14 @@
 #define PJ_LIB__
-#include <projects.h>
+#include <errno.h>
+#include <proj.h>
+#include "projects.h"
+
+enum Mode {
+    N_POLE = 0,
+    S_POLE = 1,
+    EQUIT  = 2,
+    OBLIQ  = 3
+};
 
 struct pj_opaque {
     double  height;
@@ -14,7 +23,7 @@ struct pj_opaque {
     double  sg;
     double  sw;
     double  cw;
-    int     mode;
+    enum Mode mode;
     int     tilt;
 };
 
@@ -22,10 +31,6 @@ PROJ_HEAD(nsper, "Near-sided perspective") "\n\tAzi, Sph\n\th=";
 PROJ_HEAD(tpers, "Tilted perspective") "\n\tAzi, Sph\n\ttilt= azi= h=";
 
 # define EPS10 1.e-10
-# define N_POLE 0
-# define S_POLE 1
-# define EQUIT  2
-# define OBLIQ  3
 
 
 static XY s_forward (LP lp, PJ *P) {           /* Spheroidal, forward */
@@ -50,7 +55,10 @@ static XY s_forward (LP lp, PJ *P) {           /* Spheroidal, forward */
         xy.y = sinphi;
         break;
     }
-    if (xy.y < Q->rp) F_ERROR;
+    if (xy.y < Q->rp) {
+        proj_errno_set(P, PJD_ERR_TOLERANCE_CONDITION);
+        return xy;
+    }
     xy.y = Q->pn1 / (Q->p - xy.y);
     xy.x = xy.y * cosphi * sin(lp.lam);
     switch (Q->mode) {
@@ -63,6 +71,7 @@ static XY s_forward (LP lp, PJ *P) {           /* Spheroidal, forward */
         break;
     case N_POLE:
         coslam = - coslam;
+        /*-fallthrough*/
     case S_POLE:
         xy.y *= cosphi * coslam;
         break;
@@ -94,7 +103,10 @@ static LP s_inverse (XY xy, PJ *P) {           /* Spheroidal, inverse */
         xy.y = bq * Q->cg - bm * Q->sg;
     }
     rh = hypot(xy.x, xy.y);
-    if ((sinz = 1. - rh * rh * Q->pfact) < 0.) I_ERROR;
+    if ((sinz = 1. - rh * rh * Q->pfact) < 0.) {
+        proj_errno_set(P, PJD_ERR_TOLERANCE_CONDITION);
+        return lp;
+    }
     sinz = (Q->p - sqrt(sinz)) / (Q->pn1 / rh + rh / Q->pn1);
     cosz = sqrt(1. - sinz * sinz);
     if (fabs(rh) <= EPS10) {
@@ -126,26 +138,12 @@ static LP s_inverse (XY xy, PJ *P) {           /* Spheroidal, inverse */
 }
 
 
-static void *freeup_new (PJ *P) {                       /* Destructor */
-    if (0==P)
-        return 0;
-    if (0==P->opaque)
-        return pj_dealloc (P);
-
-    pj_dealloc (P->opaque);
-    return pj_dealloc(P);
-}
-
-static void freeup (PJ *P) {
-    freeup_new (P);
-    return;
-}
-
-
 static PJ *setup(PJ *P) {
     struct pj_opaque *Q = P->opaque;
 
-    if ((Q->height = pj_param(P->ctx, P->params, "dh").f) <= 0.) E_ERROR(-30);
+    if ((Q->height = pj_param(P->ctx, P->params, "dh").f) <= 0.)
+        return pj_default_destructor(P, PJD_ERR_H_LESS_THAN_ZERO);
+
     if (fabs(fabs(P->phi0) - M_HALFPI) < EPS10)
         Q->mode = P->phi0 < 0. ? S_POLE : N_POLE;
     else if (fabs(P->phi0) < EPS10)
@@ -163,6 +161,7 @@ static PJ *setup(PJ *P) {
     P->inv = s_inverse;
     P->fwd = s_forward;
     P->es = 0.;
+
     return P;
 }
 
@@ -170,7 +169,7 @@ static PJ *setup(PJ *P) {
 PJ *PROJECTION(nsper) {
     struct pj_opaque *Q = pj_calloc (1, sizeof (struct pj_opaque));
     if (0==Q)
-        return freeup_new (P);
+        return pj_default_destructor (P, ENOMEM);
     P->opaque = Q;
 
     Q->tilt = 0;
@@ -184,7 +183,7 @@ PJ *PROJECTION(tpers) {
 
     struct pj_opaque *Q = pj_calloc (1, sizeof (struct pj_opaque));
     if (0==Q)
-        return freeup_new (P);
+        return pj_default_destructor (P, ENOMEM);
     P->opaque = Q;
 
     omega = pj_param(P->ctx, P->params, "dtilt").f * DEG_TO_RAD;
@@ -196,92 +195,3 @@ PJ *PROJECTION(tpers) {
     return setup(P);
 }
 
-
-#ifndef PJ_SELFTEST
-int pj_nsper_selftest (void) {return 0;}
-#else
-
-int pj_nsper_selftest (void) {
-    double tolerance_lp = 1e-10;
-    double tolerance_xy = 1e-7;
-
-    char s_args[] = {"+proj=nsper   +a=6400000  +h=1000000"};
-
-    LP fwd_in[] = {
-        { 2, 1},
-        { 2,-1},
-        {-2, 1},
-        {-2,-1}
-    };
-
-    XY s_fwd_expect[] = {
-        { 222239.816114099842,  111153.763991924759},
-        { 222239.816114099842, -111153.763991924759},
-        {-222239.816114099842,  111153.763991924759},
-        {-222239.816114099842, -111153.763991924759},
-    };
-
-    XY inv_in[] = {
-        { 200, 100},
-        { 200,-100},
-        {-200, 100},
-        {-200,-100}
-    };
-
-    LP s_inv_expect[] = {
-        { 0.00179049311728792437,  0.000895246558425396135},
-        { 0.00179049311728792437, -0.000895246558425396135},
-        {-0.00179049311728792437,  0.000895246558425396135},
-        {-0.00179049311728792437, -0.000895246558425396135},
-    };
-
-    return pj_generic_selftest (0, s_args, tolerance_xy, tolerance_lp, 4, 4, fwd_in, 0, s_fwd_expect, inv_in, 0, s_inv_expect);
-}
-
-
-#endif
-
-
-#ifndef PJ_SELFTEST
-int pj_tpers_selftest (void) {return 0;}
-#else
-
-int pj_tpers_selftest (void) {
-    double tolerance_lp = 1e-10;
-    double tolerance_xy = 1e-7;
-
-    char s_args[] = {"+proj=tpers   +a=6400000  +h=1000000 +azi=20"};
-
-    LP fwd_in[] = {
-        { 2, 1},
-        { 2,-1},
-        {-2, 1},
-        {-2,-1}
-    };
-
-    XY s_fwd_expect[] = {
-        { 170820.288955531199,  180460.865555804776},
-        { 246853.941538942483, -28439.8780357754222},
-        {-246853.941538942483,  28439.8780357754222},
-        {-170820.288955531199, -180460.865555804776}
-    };
-
-    XY inv_in[] = {
-        { 200, 100},
-        { 200,-100},
-        {-200, 100},
-        {-200,-100}
-    };
-
-    LP s_inv_expect[] = {
-        { 0.00198870552603137678,  0.000228871872278689991},
-        { 0.00137632081376749859, -0.00145364129728205432},
-        {-0.00137632081376749859,  0.00145364129728205432},
-        {-0.00198870552603137678, -0.000228871872278689991},
-    };
-
-    return pj_generic_selftest (0, s_args, tolerance_xy, tolerance_lp, 4, 4, fwd_in, 0, s_fwd_expect, inv_in, 0, s_inv_expect);
-}
-
-
-#endif
