@@ -5,13 +5,13 @@
 
 static double proj_strtod(char *nptr, char **endptr);
 
-/* following should be sufficient for all but the rediculous */
+/* following should be sufficient for all but the ridiculous */
 #define MAX_WORK 64
 	static const char
 *sym = "NnEeSsWw";
 	static const double
 vm[] = {
-	.0174532925199433,
+	DEG_TO_RAD,
 	.0002908882086657216,
 	.0000048481368110953599
 };
@@ -108,6 +108,6 @@ proj_strtod(char *nptr, char **endptr)
 
     /* no offending characters, just handle normally */
 
-    return strtod(nptr, endptr);
+    return pj_strtod(nptr, endptr);
 }
 
