@@ -1,7 +1,4 @@
 /* print projection's list of parameters */
-#ifndef lint
-static const char SCCSID[]="@(#)pj_pr_list.c	4.6   94/03/19 GIE REL";
-#endif
 #include <projects.h>
 #include <stdio.h>
 #include <string.h>
@@ -14,7 +11,7 @@ pr_list(PJ *P, int not_used) {
 	(void)putchar('#');
 	for (t = P->params; t; t = t->next)
 		if ((!not_used && t->used) || (not_used && !t->used)) {
-			l = strlen(t->param) + 1;
+			l = (int)strlen(t->param) + 1;
 			if (n + l > LINE_LEN) {
 				(void)fputs("\n#", stdout);
 				n = 2;
@@ -52,7 +49,7 @@ pj_pr_list(PJ *P) {
 /*                                                                      */
 /*      Returns the PROJ.4 command string that would produce this       */
 /*      definition expanded as much as possible.  For instance,         */
-/*      +init= calls and +datum= defintions would be expanded.          */
+/*      +init= calls and +datum= definitions would be expanded.         */
 /************************************************************************/
 
 char *pj_get_def( PJ *P, int options )
@@ -61,9 +58,12 @@ char *pj_get_def( PJ *P, int options )
     paralist *t;
     int l;
     char *definition;
-    int  def_max = 10;
+    size_t def_max = 10;
+    (void) options;
 
     definition = (char *) pj_malloc(def_max);
+    if (!definition)
+        return NULL;
     definition[0] = '\0';
 
     for (t = P->params; t; t = t->next)
@@ -73,16 +73,22 @@ char *pj_get_def( PJ *P, int options )
             continue;
 
         /* grow the resulting string if needed */
-        l = strlen(t->param) + 1;
+        l = (int)strlen(t->param) + 1;
         if( strlen(definition) + l + 5 > def_max )
         {
             char *def2;
 
             def_max = def_max * 2 + l + 5;
             def2 = (char *) pj_malloc(def_max);
-            strcpy( def2, definition );
-            pj_dalloc( definition );
-            definition = def2;
+            if (def2) {
+                strcpy( def2, definition );
+                pj_dalloc( definition );
+                definition = def2;
+            }
+            else {
+                pj_dalloc( definition );
+                return NULL;
+            }
         }
 
         /* append this parameter */
