@@ -1,8 +1,6 @@
 /******************************************************************************
- * $Id: pj_transform.c 2000 2011-05-10 17:06:33Z warmerdam $
- *
  * Project:  PROJ.4
- * Purpose:  Perform overall coordinate system to coordinate system 
+ * Purpose:  Perform overall coordinate system to coordinate system
  *           transformations (pj_transform() function) including reprojection
  *           and datum shifting.
  * Author:   Frank Warmerdam, warmerdam@pobox.com
@@ -34,10 +32,8 @@
 #include <math.h>
 #include "geocent.h"
 
-PJ_CVSID("$Id: pj_transform.c 2000 2011-05-10 17:06:33Z warmerdam $");
-
 static int pj_adjust_axis( projCtx ctx, const char *axis, int denormalize_flag,
-                           long point_count, int point_offset, 
+                           long point_count, int point_offset,
                            double *x, double *y, double *z );
 
 #ifndef SRS_WGS84_SEMIMAJOR
@@ -56,24 +52,34 @@ static int pj_adjust_axis( projCtx ctx, const char *axis, int denormalize_flag,
 #define Rz_BF (defn->datum_params[5])
 #define M_BF  (defn->datum_params[6])
 
-/* 
-** This table is intended to indicate for any given error code in 
-** the range 0 to -44, whether that error will occur for all locations (ie.
+/*
+** This table is intended to indicate for any given error code in
+** the range 0 to -56, whether that error will occur for all locations (ie.
 ** it is a problem with the coordinate system as a whole) in which case the
 ** value would be 0, or if the problem is with the point being transformed
-** in which case the value is 1. 
+** in which case the value is 1.
 **
 ** At some point we might want to move this array in with the error message
-** list or something, but while experimenting with it this should be fine. 
+** list or something, but while experimenting with it this should be fine.
+**
+**
+** NOTE (2017-10-01): Non-transient errors really should have resulted in a
+** PJ==0 during initialization, and hence should be handled at the level
+** before calling pj_transform. The only obvious example of the contrary
+** appears to be the PJD_ERR_GRID_AREA case, which may also be taken to
+** mean "no grids available"
+**
+**
 */
 
-static const int transient_error[50] = {
+static const int transient_error[60] = {
     /*             0  1  2  3  4  5  6  7  8  9   */
-    /* 0 to 9 */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-    /* 10 to 19 */ 0, 0, 0, 0, 1, 1, 0, 1, 1, 1,  
-    /* 20 to 29 */ 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 
-    /* 30 to 39 */ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-    /* 40 to 49 */ 0, 0, 0, 0, 0, 0, 0, 0, 1, 0 };
+    /* 0 to 9 */   0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    /* 10 to 19 */ 0, 0, 0, 0, 1, 1, 0, 1, 1, 1,
+    /* 20 to 29 */ 1, 0, 0, 0, 0, 0, 0, 1, 0, 0,
+    /* 30 to 39 */ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    /* 40 to 49 */ 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
+    /* 50 to 59 */ 1, 0, 1, 0, 1, 1, 1, 1, 0, 0 };
 
 /************************************************************************/
 /*                            pj_transform()                            */
@@ -89,7 +95,6 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
 
 {
     long      i;
-    int       err;
 
     srcdefn->ctx->last_errno = 0;
     dstdefn->ctx->last_errno = 0;
@@ -105,26 +110,18 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
     {
         int err;
 
-        err = pj_adjust_axis( srcdefn->ctx, srcdefn->axis, 
+        err = pj_adjust_axis( srcdefn->ctx, srcdefn->axis,
                               0, point_count, point_offset, x, y, z );
         if( err != 0 )
             return err;
     }
 
-/* -------------------------------------------------------------------- */
-/*      Transform Z to meters if it isn't already.                      */
-/* -------------------------------------------------------------------- */
-    if( srcdefn->vto_meter != 1.0 && z != NULL )
-    {
-        for( i = 0; i < point_count; i++ )
-            z[point_offset*i] *= srcdefn->vto_meter;
-    }
-
 /* -------------------------------------------------------------------- */
 /*      Transform geocentric source coordinates to lat/long.            */
 /* -------------------------------------------------------------------- */
     if( srcdefn->is_geocent )
     {
+        int err;
         if( z == NULL )
         {
             pj_ctx_set_errno( pj_get_ctx(srcdefn), PJD_ERR_GEOCENTRIC);
@@ -139,12 +136,13 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
                 {
                     x[point_offset*i] *= srcdefn->to_meter;
                     y[point_offset*i] *= srcdefn->to_meter;
+                    z[point_offset*i] *= srcdefn->to_meter;
                 }
             }
         }
 
         err = pj_geocentric_to_geodetic( srcdefn->a_orig, srcdefn->es_orig,
-                                         point_count, point_offset, 
+                                         point_count, point_offset,
                                          x, y, z );
         if( err != 0 )
             return err;
@@ -156,50 +154,103 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
 /* -------------------------------------------------------------------- */
     else if( !srcdefn->is_latlong )
     {
-        if( srcdefn->inv == NULL )
+
+        /* Check first if projection is invertible. */
+        if( (srcdefn->inv3d == NULL) && (srcdefn->inv == NULL))
         {
             pj_ctx_set_errno( pj_get_ctx(srcdefn), -17 );
-            pj_log( pj_get_ctx(srcdefn), PJ_LOG_ERROR, 
+            pj_log( pj_get_ctx(srcdefn), PJ_LOG_ERROR,
                     "pj_transform(): source projection not invertable" );
             return -17;
         }
 
-        for( i = 0; i < point_count; i++ )
+        /* If invertible - First try inv3d if defined */
+        if (srcdefn->inv3d != NULL)
         {
-            XY         projected_loc;
-            LP	       geodetic_loc;
-
-            projected_loc.u = x[point_offset*i];
-            projected_loc.v = y[point_offset*i];
-
-            if( projected_loc.u == HUGE_VAL )
-                continue;
-
-            geodetic_loc = pj_inv( projected_loc, srcdefn );
-            if( srcdefn->ctx->last_errno != 0 )
+            /* Three dimensions must be defined */
+            if ( z == NULL)
             {
-                if( (srcdefn->ctx->last_errno != 33 /*EDOM*/ 
-                     && srcdefn->ctx->last_errno != 34 /*ERANGE*/ )
-                    && (srcdefn->ctx->last_errno > 0 
-                        || srcdefn->ctx->last_errno < -44 || point_count == 1
-                        || transient_error[-srcdefn->ctx->last_errno] == 0 ) )
-                    return srcdefn->ctx->last_errno;
-                else
-                {
-                    geodetic_loc.u = HUGE_VAL;
-                    geodetic_loc.v = HUGE_VAL;
-                }
+                pj_ctx_set_errno( pj_get_ctx(srcdefn), PJD_ERR_GEOCENTRIC);
+                return PJD_ERR_GEOCENTRIC;
             }
 
-            x[point_offset*i] = geodetic_loc.u;
-            y[point_offset*i] = geodetic_loc.v;
+            for (i=0; i < point_count; i++)
+            {
+                XYZ projected_loc;
+                XYZ geodetic_loc;
+
+                projected_loc.u = x[point_offset*i];
+                projected_loc.v = y[point_offset*i];
+                projected_loc.w = z[point_offset*i];
+
+                if (projected_loc.u == HUGE_VAL)
+                    continue;
+
+                geodetic_loc = pj_inv3d(projected_loc, srcdefn);
+                if( srcdefn->ctx->last_errno != 0 )
+                {
+                    if( (srcdefn->ctx->last_errno != 33 /*EDOM*/
+                         && srcdefn->ctx->last_errno != 34 /*ERANGE*/ )
+                        && (srcdefn->ctx->last_errno > 0
+                            || srcdefn->ctx->last_errno < -44 || point_count == 1
+                            || transient_error[-srcdefn->ctx->last_errno] == 0 ) )
+                        return srcdefn->ctx->last_errno;
+                    else
+                    {
+                        geodetic_loc.u = HUGE_VAL;
+                        geodetic_loc.v = HUGE_VAL;
+                        geodetic_loc.w = HUGE_VAL;
+                    }
+                }
+
+                x[point_offset*i] = geodetic_loc.u;
+                y[point_offset*i] = geodetic_loc.v;
+                z[point_offset*i] = geodetic_loc.w;
+
+            }
+
+        }
+        else
+        {
+            /* Fallback to the original PROJ.4 API 2d inversion - inv */
+            for( i = 0; i < point_count; i++ )
+            {
+                XY         projected_loc;
+                LP	       geodetic_loc;
+
+                projected_loc.u = x[point_offset*i];
+                projected_loc.v = y[point_offset*i];
+
+                if( projected_loc.u == HUGE_VAL )
+                    continue;
+
+                geodetic_loc = pj_inv( projected_loc, srcdefn );
+                if( srcdefn->ctx->last_errno != 0 )
+                {
+                    if( (srcdefn->ctx->last_errno != 33 /*EDOM*/
+                         && srcdefn->ctx->last_errno != 34 /*ERANGE*/ )
+                        && (srcdefn->ctx->last_errno > 0
+                            || srcdefn->ctx->last_errno < -44 || point_count == 1
+                            || transient_error[-srcdefn->ctx->last_errno] == 0 ) )
+                        return srcdefn->ctx->last_errno;
+                    else
+                    {
+                        geodetic_loc.u = HUGE_VAL;
+                        geodetic_loc.v = HUGE_VAL;
+                    }
+                }
+
+                x[point_offset*i] = geodetic_loc.u;
+                y[point_offset*i] = geodetic_loc.v;
+            }
         }
     }
+
 /* -------------------------------------------------------------------- */
 /*      But if they are already lat long, adjust for the prime          */
 /*      meridian if there is one in effect.                             */
 /* -------------------------------------------------------------------- */
-    if( srcdefn->from_greenwich != 0.0 )
+    if ((srcdefn->is_geocent || srcdefn->is_latlong) && ( srcdefn->from_greenwich != 0.0 ))
     {
         for( i = 0; i < point_count; i++ )
         {
@@ -212,19 +263,19 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
 /*      Do we need to translate from geoid to ellipsoidal vertical      */
 /*      datum?                                                          */
 /* -------------------------------------------------------------------- */
-    if( srcdefn->has_geoid_vgrids )
+    if( srcdefn->has_geoid_vgrids && z != NULL )
     {
-        if( pj_apply_vgridshift( srcdefn, "sgeoidgrids", 
-                                 &(srcdefn->vgridlist_geoid), 
+        if( pj_apply_vgridshift( srcdefn, "sgeoidgrids",
+                                 &(srcdefn->vgridlist_geoid),
                                  &(srcdefn->vgridlist_geoid_count),
                                  0, point_count, point_offset, x, y, z ) != 0 )
             return pj_ctx_get_errno(srcdefn->ctx);
     }
-        
+
 /* -------------------------------------------------------------------- */
 /*      Convert datums if needed, and possible.                         */
 /* -------------------------------------------------------------------- */
-    if( pj_datum_transform( srcdefn, dstdefn, point_count, point_offset, 
+    if( pj_datum_transform( srcdefn, dstdefn, point_count, point_offset,
                             x, y, z ) != 0 )
     {
         if( srcdefn->ctx->last_errno != 0 )
@@ -234,31 +285,30 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
     }
 
 /* -------------------------------------------------------------------- */
-/*      Do we need to translate from geoid to ellipsoidal vertical      */
+/*      Do we need to translate from ellipsoidal to geoid vertical      */
 /*      datum?                                                          */
 /* -------------------------------------------------------------------- */
-    if( dstdefn->has_geoid_vgrids )
+    if( dstdefn->has_geoid_vgrids && z != NULL )
     {
-        if( pj_apply_vgridshift( dstdefn, "sgeoidgrids", 
-                                 &(dstdefn->vgridlist_geoid), 
+        if( pj_apply_vgridshift( dstdefn, "sgeoidgrids",
+                                 &(dstdefn->vgridlist_geoid),
                                  &(dstdefn->vgridlist_geoid_count),
                                  1, point_count, point_offset, x, y, z ) != 0 )
             return dstdefn->ctx->last_errno;
     }
-        
+
 /* -------------------------------------------------------------------- */
 /*      But if they are staying lat long, adjust for the prime          */
 /*      meridian if there is one in effect.                             */
 /* -------------------------------------------------------------------- */
-    if( dstdefn->from_greenwich != 0.0 )
+    if ((dstdefn->is_geocent || dstdefn->is_latlong) && ( dstdefn->from_greenwich != 0.0 ))
     {
         for( i = 0; i < point_count; i++ )
         {
             if( x[point_offset*i] != HUGE_VAL )
                 x[point_offset*i] -= dstdefn->from_greenwich;
         }
-    }
-
+}
 
 /* -------------------------------------------------------------------- */
 /*      Transform destination latlong to geocentric if required.        */
@@ -282,6 +332,7 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
                 {
                     x[point_offset*i] *= dstdefn->fr_meter;
                     y[point_offset*i] *= dstdefn->fr_meter;
+                    z[point_offset*i] *= srcdefn->fr_meter;
                 }
             }
         }
@@ -293,35 +344,83 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
 /* -------------------------------------------------------------------- */
     else if( !dstdefn->is_latlong )
     {
-        for( i = 0; i < point_count; i++ )
+
+        if( dstdefn->fwd3d != NULL)
         {
-            XY         projected_loc;
-            LP	       geodetic_loc;
-
-            geodetic_loc.u = x[point_offset*i];
-            geodetic_loc.v = y[point_offset*i];
-
-            if( geodetic_loc.u == HUGE_VAL )
-                continue;
-
-            projected_loc = pj_fwd( geodetic_loc, dstdefn );
-            if( dstdefn->ctx->last_errno != 0 )
+            /* Three dimensions must be defined */
+            if ( z == NULL)
             {
-                if( (dstdefn->ctx->last_errno != 33 /*EDOM*/ 
-                     && dstdefn->ctx->last_errno != 34 /*ERANGE*/ )
-                    && (dstdefn->ctx->last_errno > 0 
-                        || dstdefn->ctx->last_errno < -44 || point_count == 1
-                        || transient_error[-dstdefn->ctx->last_errno] == 0 ) )
-                    return dstdefn->ctx->last_errno;
-                else
-                {
-                    projected_loc.u = HUGE_VAL;
-                    projected_loc.v = HUGE_VAL;
-                }
+                pj_ctx_set_errno( pj_get_ctx(dstdefn), PJD_ERR_GEOCENTRIC);
+                return PJD_ERR_GEOCENTRIC;
             }
 
-            x[point_offset*i] = projected_loc.u;
-            y[point_offset*i] = projected_loc.v;
+            for( i = 0; i < point_count; i++ )
+            {
+                XYZ projected_loc;
+                LPZ geodetic_loc;
+
+                geodetic_loc.u = x[point_offset*i];
+                geodetic_loc.v = y[point_offset*i];
+                geodetic_loc.w = z[point_offset*i];
+
+                if (geodetic_loc.u == HUGE_VAL)
+                    continue;
+
+                projected_loc = pj_fwd3d( geodetic_loc, dstdefn);
+                if( dstdefn->ctx->last_errno != 0 )
+                {
+                    if( (dstdefn->ctx->last_errno != 33 /*EDOM*/
+                         && dstdefn->ctx->last_errno != 34 /*ERANGE*/ )
+                        && (dstdefn->ctx->last_errno > 0
+                            || dstdefn->ctx->last_errno < -44 || point_count == 1
+                            || transient_error[-dstdefn->ctx->last_errno] == 0 ) )
+                        return dstdefn->ctx->last_errno;
+                    else
+                    {
+                        projected_loc.u = HUGE_VAL;
+                        projected_loc.v = HUGE_VAL;
+                        projected_loc.w = HUGE_VAL;
+                    }
+                }
+
+                x[point_offset*i] = projected_loc.u;
+                y[point_offset*i] = projected_loc.v;
+                z[point_offset*i] = projected_loc.w;
+            }
+
+        }
+        else
+        {
+            for( i = 0; i < point_count; i++ )
+            {
+                XY         projected_loc;
+                LP	       geodetic_loc;
+
+                geodetic_loc.u = x[point_offset*i];
+                geodetic_loc.v = y[point_offset*i];
+
+                if( geodetic_loc.u == HUGE_VAL )
+                    continue;
+
+                projected_loc = pj_fwd( geodetic_loc, dstdefn );
+                if( dstdefn->ctx->last_errno != 0 )
+                {
+                    if( (dstdefn->ctx->last_errno != 33 /*EDOM*/
+                         && dstdefn->ctx->last_errno != 34 /*ERANGE*/ )
+                        && (dstdefn->ctx->last_errno > 0
+                            || dstdefn->ctx->last_errno < -44 || point_count == 1
+                            || transient_error[-dstdefn->ctx->last_errno] == 0 ) )
+                        return dstdefn->ctx->last_errno;
+                    else
+                    {
+                        projected_loc.u = HUGE_VAL;
+                        projected_loc.v = HUGE_VAL;
+                    }
+                }
+
+                x[point_offset*i] = projected_loc.u;
+                y[point_offset*i] = projected_loc.v;
+            }
         }
     }
 
@@ -333,25 +432,20 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
     {
         for( i = 0; i < point_count; i++ )
         {
-            if( x[point_offset*i] == HUGE_VAL )
+            double val = x[point_offset*i];
+            if( val == HUGE_VAL )
                 continue;
 
-            while( x[point_offset*i] < dstdefn->long_wrap_center - PI )
-                x[point_offset*i] += TWOPI;
-            while( x[point_offset*i] > dstdefn->long_wrap_center + PI )
-                x[point_offset*i] -= TWOPI;
+            /* Get fast in ] -2 PI, 2 PI [ range */
+            val = fmod(val, M_TWOPI);
+            while( val < dstdefn->long_wrap_center - M_PI )
+                val += M_TWOPI;
+            while( val > dstdefn->long_wrap_center + M_PI )
+                val -= M_TWOPI;
+            x[point_offset*i] = val;
         }
     }
 
-/* -------------------------------------------------------------------- */
-/*      Transform Z from meters if needed.                              */
-/* -------------------------------------------------------------------- */
-    if( dstdefn->vto_meter != 1.0 && z != NULL )
-    {
-        for( i = 0; i < point_count; i++ )
-            z[point_offset*i] *= dstdefn->vfr_meter;
-    }
-
 /* -------------------------------------------------------------------- */
 /*      Transform normalized axes into unusual output coordinate axis   */
 /*      orientation if needed.                                          */
@@ -360,7 +454,7 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
     {
         int err;
 
-        err = pj_adjust_axis( dstdefn->ctx, dstdefn->axis, 
+        err = pj_adjust_axis( dstdefn->ctx, dstdefn->axis,
                               1, point_count, point_offset, x, y, z );
         if( err != 0 )
             return err;
@@ -373,7 +467,7 @@ int pj_transform( PJ *srcdefn, PJ *dstdefn, long point_count, int point_offset,
 /*                     pj_geodetic_to_geocentric()                      */
 /************************************************************************/
 
-int pj_geodetic_to_geocentric( double a, double es, 
+int pj_geodetic_to_geocentric( double a, double es,
                                long point_count, int point_offset,
                                double *x, double *y, double *z )
 
@@ -400,7 +494,7 @@ int pj_geodetic_to_geocentric( double a, double es,
         if( x[io] == HUGE_VAL  )
             continue;
 
-        if( pj_Convert_Geodetic_To_Geocentric( &gi, y[io], x[io], z[io], 
+        if( pj_Convert_Geodetic_To_Geocentric( &gi, y[io], x[io], z[io],
                                                x+io, y+io, z+io ) != 0 )
         {
             ret_errno = -14;
@@ -416,7 +510,7 @@ int pj_geodetic_to_geocentric( double a, double es,
 /*                     pj_geodetic_to_geocentric()                      */
 /************************************************************************/
 
-int pj_geocentric_to_geodetic( double a, double es, 
+int pj_geocentric_to_geodetic( double a, double es,
                                long point_count, int point_offset,
                                double *x, double *y, double *z )
 
@@ -442,7 +536,7 @@ int pj_geocentric_to_geodetic( double a, double es,
         if( x[io] == HUGE_VAL )
             continue;
 
-        pj_Convert_Geocentric_To_Geodetic( &gi, x[io], y[io], z[io], 
+        pj_Convert_Geocentric_To_Geodetic( &gi, x[io], y[io], z[io],
                                            y+io, x+io, z+io );
     }
 
@@ -463,10 +557,10 @@ int pj_compare_datums( PJ *srcdefn, PJ *dstdefn )
     {
         return 0;
     }
-    else if( srcdefn->a_orig != dstdefn->a_orig 
+    else if( srcdefn->a_orig != dstdefn->a_orig
              || ABS(srcdefn->es_orig - dstdefn->es_orig) > 0.000000000050 )
     {
-        /* the tolerence for es is to ensure that GRS80 and WGS84 are
+        /* the tolerance for es is to ensure that GRS80 and WGS84 are
            considered identical */
         return 0;
     }
@@ -488,8 +582,12 @@ int pj_compare_datums( PJ *srcdefn, PJ *dstdefn )
     }
     else if( srcdefn->datum_type == PJD_GRIDSHIFT )
     {
-        return strcmp( pj_param(srcdefn->ctx, srcdefn->params,"snadgrids").s,
-                       pj_param(dstdefn->ctx, dstdefn->params,"snadgrids").s ) == 0;
+        const char* srcnadgrids =
+            pj_param(srcdefn->ctx, srcdefn->params,"snadgrids").s;
+        const char* dstnadgrids =
+            pj_param(dstdefn->ctx, dstdefn->params,"snadgrids").s;
+        return srcnadgrids != 0 && dstnadgrids != 0 &&
+               strcmp( srcnadgrids, dstnadgrids ) == 0;
     }
     else
         return 1;
@@ -499,7 +597,8 @@ int pj_compare_datums( PJ *srcdefn, PJ *dstdefn )
 /*                       pj_geocentic_to_wgs84()                        */
 /************************************************************************/
 
-int pj_geocentric_to_wgs84( PJ *defn, 
+static
+int pj_geocentric_to_wgs84( PJ *defn,
                             long point_count, int point_offset,
                             double *x, double *y, double *z )
 
@@ -511,7 +610,7 @@ int pj_geocentric_to_wgs84( PJ *defn,
         for( i = 0; i < point_count; i++ )
         {
             long io = i * point_offset;
-            
+
             if( x[io] == HUGE_VAL )
                 continue;
 
@@ -547,7 +646,8 @@ int pj_geocentric_to_wgs84( PJ *defn,
 /*                      pj_geocentic_from_wgs84()                       */
 /************************************************************************/
 
-int pj_geocentric_from_wgs84( PJ *defn, 
+static
+int pj_geocentric_from_wgs84( PJ *defn,
                               long point_count, int point_offset,
                               double *x, double *y, double *z )
 
@@ -562,7 +662,7 @@ int pj_geocentric_from_wgs84( PJ *defn,
 
             if( x[io] == HUGE_VAL )
                 continue;
-            
+
             x[io] = x[io] - Dx_BF;
             y[io] = y[io] - Dy_BF;
             z[io] = z[io] - Dz_BF;
@@ -599,7 +699,7 @@ int pj_geocentric_from_wgs84( PJ *defn,
 /*      coordinates in radians in the destination datum.                */
 /************************************************************************/
 
-int pj_datum_transform( PJ *srcdefn, PJ *dstdefn, 
+int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
                         long point_count, int point_offset,
                         double *x, double *y, double *z )
 
@@ -648,32 +748,47 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
 /* -------------------------------------------------------------------- */
     if( srcdefn->datum_type == PJD_GRIDSHIFT )
     {
+        const char* srcnadgrids = pj_param(srcdefn->ctx, srcdefn->params,"snadgrids").s;
+
         pj_apply_gridshift_2( srcdefn, 0, point_count, point_offset, x, y, z );
         CHECK_RETURN(srcdefn);
 
-        src_a = SRS_WGS84_SEMIMAJOR;
-        src_es = SRS_WGS84_ESQUARED;
+        /* If the gridlist has either "@null" or "null" as its only    */
+        /* grid we don't change the ellipsoid parameters, since the    */
+        /* datum shift to WGS84 was not performed in practice.         */
+        if ( srcnadgrids != NULL &&
+             strcmp("@null", srcnadgrids) && strcmp("null", srcnadgrids) ) {
+            src_a = SRS_WGS84_SEMIMAJOR;
+            src_es = SRS_WGS84_ESQUARED;
+        }
     }
 
     if( dstdefn->datum_type == PJD_GRIDSHIFT )
     {
-        dst_a = SRS_WGS84_SEMIMAJOR;
-        dst_es = SRS_WGS84_ESQUARED;
+        const char* dstnadgrids = pj_param(dstdefn->ctx, dstdefn->params,"snadgrids").s;
+        /* If the gridlist has either "@null" or "null" as its only    */
+        /* grid we don't change the ellipsoid parameters, since the    */
+        /* datum shift to WGS84 will not be performed.                 */
+        if ( dstnadgrids != NULL &&
+             strcmp("@null", dstnadgrids) && strcmp("null", dstnadgrids) ) {
+            dst_a = SRS_WGS84_SEMIMAJOR;
+            dst_es = SRS_WGS84_ESQUARED;
+          }
     }
 
 /* ==================================================================== */
 /*      Do we need to go through geocentric coordinates?                */
 /* ==================================================================== */
     if( src_es != dst_es || src_a != dst_a
-        || srcdefn->datum_type == PJD_3PARAM 
+        || srcdefn->datum_type == PJD_3PARAM
         || srcdefn->datum_type == PJD_7PARAM
-        || dstdefn->datum_type == PJD_3PARAM 
+        || dstdefn->datum_type == PJD_3PARAM
         || dstdefn->datum_type == PJD_7PARAM)
     {
 /* -------------------------------------------------------------------- */
 /*      Convert to geocentric coordinates.                              */
 /* -------------------------------------------------------------------- */
-        srcdefn->ctx->last_errno = 
+        srcdefn->ctx->last_errno =
             pj_geodetic_to_geocentric( src_a, src_es,
                                        point_count, point_offset, x, y, z );
         CHECK_RETURN(srcdefn);
@@ -681,14 +796,14 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
 /* -------------------------------------------------------------------- */
 /*      Convert between datums.                                         */
 /* -------------------------------------------------------------------- */
-        if( srcdefn->datum_type == PJD_3PARAM 
+        if( srcdefn->datum_type == PJD_3PARAM
             || srcdefn->datum_type == PJD_7PARAM )
         {
             pj_geocentric_to_wgs84( srcdefn, point_count, point_offset,x,y,z);
             CHECK_RETURN(srcdefn);
         }
 
-        if( dstdefn->datum_type == PJD_3PARAM 
+        if( dstdefn->datum_type == PJD_3PARAM
             || dstdefn->datum_type == PJD_7PARAM )
         {
             pj_geocentric_from_wgs84( dstdefn, point_count,point_offset,x,y,z);
@@ -698,7 +813,7 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
 /* -------------------------------------------------------------------- */
 /*      Convert back to geodetic coordinates.                           */
 /* -------------------------------------------------------------------- */
-        dstdefn->ctx->last_errno = 
+        dstdefn->ctx->last_errno =
             pj_geocentric_to_geodetic( dst_a, dst_es,
                                        point_count, point_offset, x, y, z );
         CHECK_RETURN(dstdefn);
@@ -725,9 +840,9 @@ int pj_datum_transform( PJ *srcdefn, PJ *dstdefn,
 /*      Normalize or de-normalized the x/y/z axes.  The normal form     */
 /*      is "enu" (easting, northing, up).                               */
 /************************************************************************/
-static int pj_adjust_axis( projCtx ctx, 
+static int pj_adjust_axis( projCtx ctx,
                            const char *axis, int denormalize_flag,
-                           long point_count, int point_offset, 
+                           long point_count, int point_offset,
                            double *x, double *y, double *z )
 
 {
@@ -742,7 +857,7 @@ static int pj_adjust_axis( projCtx ctx,
             y_in = y[point_offset*i];
             if( z )
                 z_in = z[point_offset*i];
-     
+
             for( i_axis = 0; i_axis < 3; i_axis++ )
             {
                 double value;
@@ -753,21 +868,29 @@ static int pj_adjust_axis( projCtx ctx,
                     value = y_in;
                 else
                     value = z_in;
-                
+
                 switch( axis[i_axis] )
                 {
                   case 'e':
-                    x[point_offset*i] = value; break;
+                    x[point_offset*i] = value;
+                    break;
                   case 'w':
-                    x[point_offset*i] = -value; break;
+                    x[point_offset*i] = -value;
+                    break;
                   case 'n':
-                    y[point_offset*i] = value; break;
+                    y[point_offset*i] = value;
+                    break;
                   case 's':
-                    y[point_offset*i] = -value; break;
+                    y[point_offset*i] = -value;
+                    break;
                   case 'u':
-                    if( z ) z[point_offset*i] = value; break;
+                    if( z )
+                        z[point_offset*i] = value;
+                    break;
                   case 'd':
-                    if( z ) z[point_offset*i] = -value; break;
+                    if( z )
+                        z[point_offset*i] = -value;
+                    break;
                   default:
                     pj_ctx_set_errno( ctx, PJD_ERR_AXIS );
                     return PJD_ERR_AXIS;
@@ -784,7 +907,7 @@ static int pj_adjust_axis( projCtx ctx,
             y_in = y[point_offset*i];
             if( z )
                 z_in = z[point_offset*i];
-     
+
             for( i_axis = 0; i_axis < 3; i_axis++ )
             {
                 double *target;
@@ -798,7 +921,7 @@ static int pj_adjust_axis( projCtx ctx,
                     target = y;
                 else
                     target = z;
-                
+
                 switch( axis[i_axis] )
                 {
                   case 'e':
@@ -820,7 +943,6 @@ static int pj_adjust_axis( projCtx ctx,
             } /* i_axis */
         } /* i (point) */
     }
-    
+
     return 0;
 }
-
