@@ -1,7 +1,4 @@
 /* Convert radian argument to DMS ascii format */
-#ifndef lint
-static const char SCCSID[]="@(#)rtodms.c	4.3	93/06/12	GIE	REL";
-#endif
 #include <projects.h>
 #include <stdio.h>
 #include <string.h>
@@ -13,7 +10,7 @@ static const char SCCSID[]="@(#)rtodms.c	4.3	93/06/12	GIE	REL";
 	static double
 RES = 1000.,
 RES60 = 60000.,
-CONV = 206264806.24709635515796003417;
+CONV = 206264806.24709635516;
 	static char
 format[50] = "%dd%d'%.3f\"%c";
 	static int
@@ -28,7 +25,7 @@ set_rtodms(int fract, int con_w) {
 		for (i = 0; i < fract; ++i)
 			RES *= 10.;
 		RES60 = RES * 60.;
-		CONV = 180. * 3600. * RES / PI;
+		CONV = 180. * 3600. * RES / M_PI;
 		if (! con_w)
 			(void)sprintf(format,"%%dd%%d'%%.%df\"%%c", fract);
 		else
@@ -52,21 +49,23 @@ rtodms(char *s, double r, int pos, int neg) {
 	r = floor(r * CONV + .5);
 	sec = fmod(r / RES, 60.);
 	r = floor(r / RES60);
-	min = fmod(r, 60.);
-        r = floor(r / 60.);
-        deg = r;
+	min = (int)fmod(r, 60.);
+	r = floor(r / 60.);
+	deg = (int)r;
 
 	if (dolong)
 		(void)sprintf(ss,format,deg,min,sec,sign);
-	else if (sec) {
+	else if (sec != 0.0) {
 		char *p, *q;
+		/* double prime + pos/neg suffix (if included) + NUL */
+		size_t suffix_len = sign ? 3 : 2;
 
 		(void)sprintf(ss,format,deg,min,sec,sign);
-		for (q = p = ss + strlen(ss) - (sign ? 3 : 2); *p == '0'; --p) ;
+		for (q = p = ss + strlen(ss) - suffix_len; *p == '0'; --p) ;
 		if (*p != '.')
 			++p;
 		if (++q != p)
-			(void)strcpy(p, q);
+			(void)memmove(p, q, suffix_len);
 	} else if (min)
 		(void)sprintf(ss,"%dd%d'%c",deg,min,sign);
 	else
